/* 
** include/wiregl_pipe.h 
** http://graphics.stanford.edu/software/wiregl
**
** Copyright 2000
** The Board of Trustees of The Leland Stanford Junior University.
** All rights reserved.
**
** Except for commercial resale, lease, license or other commercial
** transactions, permission is hereby given to use, copy, and/or
** modify this software, provided that the above copyright notice and
** this permission notice appear in all copies of this software.  No
** part of this software or any derivatives thereof may be used in
** graphics systems for resale or for use in a commercial product.
**
** This software is provided "as is" and without warranty of any kind,
** express, implied or otherwise, including without limitation, any
** warranty of merchantability or fitness for a particular purpose.
*/

#ifndef WIREGL_PIPE_H
#define WIREGL_PIPE_H

#ifdef WINDOWS
#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#endif

#ifndef NO_INCLUDE_GLCONTEXT_STUFF
#include "wiregl/glcontext/glcontext.h"
#else
struct GLcontext;
#endif

#include "wiregl/include/wiregl_util.h"
#include "wiregl/include/wiregl_protocol.h"
#include "wiregl/include/wiregl_feather.h"
#include "wiregl/include/wiregl_opcodes.h"

#if defined( IRIX ) || defined( IRIX64 ) || defined( Linux)
#include <X11/Xlib.h>
#include <X11/Xutil.h>
typedef void *GLXContext;
#endif


typedef struct {
	WireGLSemaphore someone_connected;

	char *fname;
	unsigned short port;
	int use_l2;
	int actual_window_width, actual_window_height;
	int window_width, window_height;
	int window_x, window_y;
	unsigned int depth_bits;
	unsigned int stencil_bits;
	WireGLFeatherParam feather;
	WireGLConnection *barrier_conn;
	int swap_barrier;
#ifdef WINDOWS
	HWND hWnd;
	HGLRC hRC;
	HDC device_context;
#else
	Display *dpy;
	XVisualInfo *vi;
	GLXContext cx;
	Window win;
#endif
} WireGLPipe;

extern WireGLPipe wiregl_pipe;

typedef struct WireGLReceiveBuffer {
	WireGLFreeFunc freeFunc;
	unsigned int   length;
	void          *buffer;

	struct WireGLReceiveBuffer *next;
} WireGLReceiveBuffer;

typedef struct WireGLWorkQueue {
	WireGLLock lock;
	GLcontext *context;
	int total_length;
	WireGLReceiveBuffer *head;
	WireGLReceiveBuffer *tail;
	int blocked;
	int delete_when_done;
	WireGLConnection *conn;
	WireGLConnection *peer_conn;
	struct {
		unsigned int  length;
		unsigned int  max;
		void         *buffer;
	} multi;
	struct WireGLWorkQueue *next, *prev;
} WireGLWorkQueue;

WireGLWorkQueue *__currentWorkQueue( void );

extern WireGLHashTable *PAPIbarriers, *PAPIsemaphores;

void wireGLInitNormalLookupTable( void );

void wireGLPipeserverConnect( WireGLConnection *conn, WireGLConnection *peer_conn );
void wireGLPipeserverReceive( void *buf, unsigned int len, WireGLFreeFunc free );
void wireGLPipeserverDisconnect( unsigned int sender_id );

void wireGLStartConnectionThread( void );
void wireGLCreateWindow( int fullscreen_mode );
void wireGLRenderRemoteStreams( void );

void wireGLLoadTARGATexture( char *filename );

void wireGLPipeParseConfig( char *fname );

extern WireGLEvent     work_pending;
extern WireGLLock      new_client_lock;
extern WireGLSemaphore window_sema;

typedef void (*wiregl_decoder)(void);
extern wiregl_decoder wiregl_decode_functions[];
extern unsigned char *unpacker_data_ptr;

#define READ_DATA( offset, type ) \
	*( (type *) (unpacker_data_ptr + (offset)))

#ifdef WIREGL_UNALIGNED_ACCESS_OKAY
#define READ_DOUBLE( offset ) \
	READ_DATA( offset, GLdouble )
#else
#define READ_DOUBLE( offset ) \
	wireGLReadUnalignedDouble( unpacker_data_ptr + (offset) )
#endif

#define DATA_POINTER( offset, type ) \
	( (type *) (unpacker_data_ptr + (offset)) )

#define INCR_DATA_PTR( delta ) \
	unpacker_data_ptr += (delta)

#define INCR_DATA_PTR_NO_ARGS() \
	INCR_DATA_PTR( 4 )

#define INCR_VAR_PTR() \
	INCR_DATA_PTR( *((int *) unpacker_data_ptr ) )

const char *wireGLOpcodeName( WireGLOpcode opcode );
void wireGLPrintFunc( WireGLOpcode opcode, unsigned char *unpacker_data_ptr );

#endif /* WIREGL_PIPE_H */
